# Functions 

#Checks if the job submission ended correctly. 
Job_completed()
{
  if [ "$Jobs_scheduler" == "slurm" ]; then
    local id1
    id1=${1##* }
    sleep 5
    if ! scontrol show job "$id1" | grep -q 'JobState=COMPLETED'; then
      Completed=false
    else
      Completed=true
    fi	

  elif [ "$Jobs_scheduler" == "lsf" ]; then
    local id2
    id2=$(head -n1 "$1" | cut -d'<' -f2 | cut -d'>' -f1)
    sleep 5
    if ! bjobs -l "$id2" | grep -q 'Status <DONE>'; then
      Completed=false
    else
      Completed=true
    fi
  elif [ "$Jobs_scheduler" == "torque" ]; then
    local id3
    id3=$(head -n1 "$1" | awk '{ print $3 }')
    sleep 5
    if ! qstat f "$id3" | grep -q 'exit_status = 0'; then
      Completed=false
    else
      Completed=true
    fi
  fi
  

}

# Check if nemo is compiled for using extrae

Compile_extrae()
{
  cd "$dir" || (echo "Error original dir doesn't exist"; exit 1)

  #Get flag lines

  line=$(sed -n '/^%FCFLAGS /p' "$Nemo_path"/arch/arch-"${arch}".fcm)
  line2=$(sed -n '/^%FPPFLAGS /p' "$Nemo_path"/arch/arch-"${arch}".fcm)
  line3=$(sed -n '/^%LDFLAGS /p' "$Nemo_path"/arch/arch-"${arch}".fcm)


  # If -rdynamic is not there recompilation is requiered and -rdynamic added

  if ! echo "${line}"|grep -q "\-rdynamic\b"; then
    echo "-rdynamic flag not found in FCFLAGS arch-${arch}.fcm: editing arch-${arch}.fcm "
    sed -i '/^%FCFLAGS/ s/$/ -rdynamic/' "${Nemo_path}"/arch/arch-"${arch}".fcm
    compile_ext=true
  fi

  if ! echo "${line3}"|grep -q "\-export-dynamic\b"; then
    echo "-export-dynamic flag not found in LDFLAGS arch-${arch}.fcm: editing arch-${arch}.fcm "
    sed -i '/^%LDFLAGS/ s/$/ -export-dynamic/' "${Nemo_path}"/arch/arch-"${arch}".fcm
    compile_ext=true
  fi
  
   # If finstrument-functions is not there recompilation is requiered and -finstrument-functions added
  if ! echo "${line}"|grep -q "\-finstrument-functions\b"; then
    echo "-finstrument-functions flag not found in arch-${arch}.fcm: editing arch-${arch}.fcm "
    sed -i '/^%FCFLAGS/ s/$/ -finstrument-functions/' "${Nemo_path}"/arch/arch-"${arch}".fcm
    compile_ext=true
  fi

  # If -g is not there, recompilation is requiered and -g added
    if ! echo "${line}"|grep -q "\-g\b"; then
      echo "-g flag not found in arch-${arch}.fcm: editing arch-${arch}.fcm "
      sed -i '/^%FCFLAGS/ s/$/ -g /' "${Nemo_path}"/arch/arch-"${arch}".fcm
      compile_ext=true
    fi

  
  if [ "$compile" == true ]; then
      echo 'compile parameter is inicialized true'
  fi

  # If -pg is  there recompilation is requiered and -pg removed 

  sed -i 's/-pg//g' "${Nemo_path}"/arch/arch-"${arch}".fcm

  if echo "${line}"|grep -q "\-pg\b"; then
    echo "-pg flag found in FCFLAGS arch-${arch}.fcm: editing arch-${arch}.fcm "
    compile_ext=true
  fi

  if echo "${line2}"|grep -q "\-pg\b"; then
    echo "-pg flag found in FPPFLAGS arch-${arch}.fcm : editing arch-${arch}.fcm "
    compile_ext=true
  fi

  if echo "${line3}"|grep -q "\-pg\b"; then
    echo "-pg flag found in LDFLAGS arch-${arch}.fcm: editing arch-${arch}.fcm "
    compile_ext=true
  fi
   
  # If nemo executable is not on the run file compile

  if ! test -f "${Nemo_path}/cfgs/${name_cfg}/EXP00/nemo"; then
    echo "nemo executable not found in ${name_cfg}"
    compile_ext=true
  fi


  if [ "$compile" == true ] || [ "$compile_ext" == true ]; then

    echo "Compiling Nemo for EXTRAE"
    echo "Output of the compilation in compile.err and compile.out"

    printf -v workload1 "cd ${Nemo_path}\n./makenemo -r ${cfg} -n ${name_cfg} -m ${arch} -j$Jobs_n_cores $comp_cfg"
    python3 ./src/Job_Creator.py -f "compile" -j "compile" --set-core "${Jobs_n_cores}" --set-core-per-node "$Jobs_cores_per_node" -s "$Jobs_scheduler" --set-time "$time" --set-queue "$queue" -w "${workload1}"

    state1=$("$job" --wait compile."$Jobs_scheduler")
    echo
    Job_completed "$state1"
    if [ $Completed == false ]; then
      echo "Nemo compilation failed, remember to load all the needed modules. Check the details in compile.err"
      echo
      exit 1
    else 
      echo "Nemo compilation successful"
      echo
    fi
    
  else
    echo "Compilation not needed"  
    echo
  fi

  #Copy all the EXP00 data but don't overwrite namelist just the executable
  cd "$dir" ||( echo "Error original dir doesn't exist"; exit 1 )
  cp -n  "${Nemo_path}"/cfgs/"${name_cfg}"/EXP00/* Run_NEMO
  cp "${Nemo_path}"/cfgs/"${name_cfg}"/EXP00/nemo Run_NEMO
  cd Run_NEMO||( echo "Error Run_NEMO folder doesn't exists";  exit 1 )
}

#Check if Nemo is compiled for using Gprof

Compile_gprof()
{

  cd "$dir" || (echo "Error original dir doesn't exist"; exit 1)

  if [ "$compile" == true ]; then
    echo 'compile parameter is inicialized true'
  fi

  # Checking if Gprof_arch file is present
  if ! test -f "${Nemo_path}/arch/arch-${arch}_GPROF.fcm"; then
    cp "${Nemo_path}"/arch/arch-"${arch}".fcm "${Nemo_path}"/arch/arch-"${arch}"_GPROF.fcm
  fi

  # Get text lines corresponding to the flags
  line=$(sed -n '/^%FCFLAGS /p' "$Nemo_path"/arch/arch-"${arch}"_GPROF.fcm)
  line2=$(sed -n '/^%FPPFLAGS /p' "$Nemo_path"/arch/arch-"${arch}"_GPROF.fcm)
  line3=$(sed -n '/^%LDFLAGS /p' "$Nemo_path"/arch/arch-"${arch}"_GPROF.fcm)
  
  # If -g is not there, recompilation is requiered and -g added
  if ! echo "${line}"|grep -q "\-g\b"; then
    echo "-g flag not found in arch-${arch}_GPROF.fcm: editing arch-${arch}_GPROF.fcm "
    sed -i '/^%FCFLAGS/ s/$/ -g /' "${Nemo_path}"/arch/arch-"${arch}"_GPROF.fcm
    compile_gprof=true
  fi

  # If -pg is not there recompilation is requiered and -pg added

  if ! echo "${line}"|grep -q "\-pg\b"; then
    echo "-pg flag not found in FCFLAGS arch-${arch}_GPROF.fcm: editing arch-${arch}_GPROF.fcm "
    sed -i '/^%FCFLAGS/ s/$/ -pg/' "${Nemo_path}"/arch/arch-"${arch}"_GPROF.fcm
    compile_gprof=true
  fi

  if ! echo "${line2}"|grep -q "\-pg\b"; then
    echo "-pg flag not found in FPPFLAGS arch-${arch}_GPROF.fcm : editing arch-${arch}_GPROF.fcm "
    sed -i '/^%FPPFLAGS/ s/$/ -pg/' "${Nemo_path}"/arch/arch-"${arch}"_GPROF.fcm
    compile_gprof=true
  fi

  if ! echo "${line3}"|grep -q "\-pg\b"; then
    echo "-pg flag not found in LDFLAGS arch-${arch}_GPROF.fcm: editing arch-${arch}_GPROF.fcm "
    sed -i '/^%LDFLAGS/ s/$/ -pg/' "${Nemo_path}"/arch/arch-"${arch}"_GPROF.fcm
    compile_gprof=true
  fi

  # If nemo executable is not on the run file compile

  if ! test -f "${Nemo_path}/cfgs/${name_cfg}_GPROF/EXP00/nemo"; then
    echo "nemo executable not found in ${name_cfg}_GPROF"
    compile_gprof=true
  fi

  if [ "$compile" == true ] || [ "$compile_gprof" == true ]; then
    echo "Compiling Nemo for GPROF"
    echo "Output of the compilation in compile.err and compile.out"

    printf -v workload1 "cd ${Nemo_path}\n./makenemo -r ${cfg} -n ${name_cfg}_GPROF -m ${arch}_GPROF -j$Jobs_n_cores $comp_cfg"
    python3 ./src/Job_Creator.py -f "compile" -j "compile" --set-core "${Jobs_n_cores}" --set-core-per-node "$Jobs_cores_per_node" -s "$Jobs_scheduler" --set-time "$time" --set-queue "$queue" -w "${workload1}"
  
    state1=$("$job" --wait compile."$Jobs_scheduler")
    echo
    Job_completed "$state1"
    if [ "$Completed" == false ]; then
      echo "Nemo compilation failed, remember to load all the needed modules. Check the details in compile.err"
      echo
      exit 1
    else 
      echo "Nemo compilation successful"
      echo
    fi
    
  else
    echo "Compilation not needed"  
    echo
  fi


  #Copy all the EXP00 data in RUN_NEMO folder but don't overwrite the namelist, just the executable.
 
  cp -n  "${Nemo_path}"/cfgs/"${name_cfg}"_GPROF/EXP00/* Run_NEMO
  cp "${Nemo_path}"/cfgs/"${name_cfg}"_GPROF/EXP00/nemo Run_NEMO
  

  if [[ $comp_cfg == "-d OCE del_key 'key_si3 key_top'" ]]; then
    sed -i '/_def_nemo-ice.xml\|def_nemo-pisces.xml/d' Run_NEMO/context_nemo.xml #DELETE ICE AND PISCES CONTEXT (NOT USED)
  fi

  #Solving NEMO input file common errors

  if test -f "Run_NEMO/weights_core_orca2_bicubic_noc.nc"; then
    mv Run_NEMO/weights_core_orca2_bicubic_noc.nc Run_NEMO/weights_core2_orca2_bicub.nc #RENAME WRONG NAMED FILES
  fi

  if test -f "weights_core_orca2_bilinear_noc.nc"; then 
    mv Run_NEMO/weights_core_orca2_bilinear_noc.nc Run_NEMO/weights_core2_orca2_bilin.nc #RENAME WRONG NAMED FILES
  fi
  
    cd Run_NEMO||(echo "Error Run_NEMO folder doesn't exists";  exit 1)

}




#Init variables with default values in case of missing

Init()
{
  Nemo_path="${Nemo_path:-"./"}"
  Nemo_input_data="${Nemo_input_data=:-"./"}"
  Nemo_cores="${Nemo_cores:-( 48 )}"
  Jobs_n_cores="${Jobs_n_cores:-48}"
  Jobs_cores_per_node="${Jobs_cores_per_node:-0}"
  Jobs_scheduler="${Jobs_scheduler:-"slurm"}"
  time="${Jobs_time:-0}"
  queue="${Jobs_queue:-""}"
  compile="${Compilation_compile:-"false"}"
  cfg="${Compilation_ref:-"ORCA2_ICE_PISCES"}"
  arch="${Compilation_arch:-"X64_MN4"}"
  name_cfg="${Compilation_name:-"ORCA2_EXTRAE"}"
  comp_cfg="${Compilation_sub:-""}"
  Modules="${Modules:-""}"
  Nemo_iterations=12
  compile_ext=false
  compile_gprof=false
}


# Checks if the paths given are correct.

Test_arguments()
{
  # Nemo path correct?
  if ! test -d "${Nemo_path}"; then
    echo "Nemo relative path: ${Nemo_path} is not found"
    echo
    exit 1
  fi


   # Is Nemo data path correct?
  if ! test -d "${Nemo_input_data}"; then
    echo "Nemo input data path: ${Nemo_input_data} is not found"
    echo
    exit 1
  
  else
    if ! test -d "Run_NEMO"; then
      mkdir Run_NEMO
    fi
    cp -P "${Nemo_input_data}"* Run_NEMO
  fi

  #Nemo_cores is array?
  if [[ ! "$(declare -p Nemo_cores)" =~ "declare -a" ]]; then
    echo "Error Nemo_cores has to be a bash array like ( 4 24 48 )"
    echo
    exit 1
  fi
  
  
  #Nemo_cores contains ints?
  
  re='^[0-9]+$'
  for core in "${Nemo_cores[@]}"
  do

    if ! [[ $core =~ $re ]] ; then
      echo "Error Nemo_cores has to contain integer values"
      echo
      exit 1
    fi
  done

  # cfg exists?
  if ! test -d "${Nemo_path}/cfgs/${cfg}"; then
    echo "configuration: ${cfg} doesn't exists in ${Nemo_path}/cfgs dir"
    echo
    exit 1
  fi
  # arch exists?
  if ! test -f "${Nemo_path}/arch/arch-${arch}.fcm"; then
    echo "architecture: arch-${arch}.fcm doesn't exists in ${Nemo_path}/arch dir"
    echo
    exit 1
  fi
  # scheduler correct?
  if [ "$Jobs_scheduler" != "slurm" ] && [ "$Jobs_scheduler" != "lsf" ] && [ "$Jobs_scheduler" != "torque" ]; then
    echo "$Jobs_scheduler is not a valid scheduler"
    echo
    exit 1
  fi
  #Nemo_cores is array?
  if [[ ! "$(declare -p Nemo_cores)" =~ "declare -a" ]]; then
    echo "Error, variable Nemo_cores has to be an array"
    echo
    exit 1
  fi
  #Modules available
  if ! module load $Modules;then
    echo "Error loading modules aborting"
    echo
    exit 1
  fi
  echo
  #$EXTRAE_HOME loaded ?
  if ! test -d "${EXTRAE_HOME}"; then
  echo "Extrae relative path: ${EXTRAE_HOME} is not found"
    echo
    exit 1
  else
    sed -i 's|home=.*|home="'"$EXTRAE_HOME"'"|g' src/extrae.xml
  fi

  # Adding -d to variable if not empty
  if [ -n "$comp_cfg" ]; then
    comp_cfg="-d $comp_cfg"
  fi
  
  # Creating auxiliar vars for submiting jobs
  if [ "$Jobs_scheduler" == "slurm" ]; then 
    job="sbatch"
  elif [ "$Jobs_scheduler" == "lsf" ]; then
    job="bsub"
  elif [ "$Jobs_scheduler" == "torque" ]; then
    job="qsub"
  fi

  
}


# Generates a list of Nemo functions 

Gprof_functions()
{

  cd Run_NEMO||(echo "Error Run_NEMO folder doesn't exists";  exit 1)
  #Generating function list in case of missing
  if ! test -f "extrae_functions.txt"; then

    Compile_gprof
    #Changing iterations, big traces generate problems.
    sed -i "s|nn_itend * =.*|nn_itend    =       $Nemo_iterations  !  last  time step (std 5475)|g" namelist_cfg
    rm gmon* 2> /dev/null
    echo "Runing Nemo with 2 cores to obtain function data..."
    echo
    python3 ./../src/Job_Creator.py -f "run" -j "run" --set-core 4 --set-core-per-node "$Jobs_cores_per_node" -s "$Jobs_scheduler" --set-time "$time" --set-queue "$queue" -w "mpirun -np 2 ./nemo" 
    
    state2=$("$job" --wait run."$Jobs_scheduler")
    Job_completed "$state2"
    if [ $Completed == false ]; then
      echo "Nemo execution failed look at Run_NEMO/run.err and Run_NEMO/ocean.output for more info"
      echo "Remember that the namelist files copied are the default ones, change theme in order to fit with the input files in the dir " 
      echo
      exit 1
    else 
      echo "Gprof files generated "
      echo
    fi
    echo "Gthrottling functions ..."
    echo
    python3 ./../src/Job_Creator.py -f "gthrottling" -j "gthrottling" --set-core 4 --set-core-per-node "$Jobs_cores_per_node" -s "$Jobs_scheduler" --set-time "$time" --set-queue "$queue" -w "./../src/gthrottling.sh nemo" 
    state3=$("$job" --wait gthrottling."$Jobs_scheduler")
    Job_completed "$state3"
    if [ $Completed == false ]; then
      echo "Error listing functions, look at Run_NEMO/gthrottling.err for more info"
      echo
      exit 1
    else
      echo "Functions listed correctly"
      echo
    fi

    
    

  else
    echo "Functions already listed, file Run_NEMO/extrae_functions.txt does exist"
    echo
    
  fi
  
}


#Gets a trace from Nemo and cuts it to obtain a single timestep

Get_trace()
{

Compile_extrae

# Run nemo with extrae
  ./../src/extraf.sh  nemo extrae_functions.txt
  sed -i "s|list=.*|list=\"${dir}/Run_NEMO/extrae_functions_for_xml.txt\" exclude-automatic-functions=\"yes\">|g" ../src/extrae.xml
  for core in "${Nemo_cores[@]}" 
  
  do
    
    echo "Creating trace with $core cores..."
    echo
    python3 ./../src/Job_Creator.py -f "run_extrae" -j "run_extrae" --set-core "$core" --set-core-per-node "$Jobs_cores_per_node" -s "$Jobs_scheduler" --set-time "$time" --set-queue "$queue" -w "mpirun -np $core ./../src/trace.sh ./nemo" 
    
    state4=$("$job" --wait run_extrae."$Jobs_scheduler")
    Job_completed "$state4"
    if [ $Completed == false ]; then
      echo "Nemo execution failed, no traces files generated more info inside Run_NEMO/run_extrae.err"
      echo 
      exit 1
    fi
    mv nemo.prv nemo_"$core".prv
    mv nemo.pcf nemo_"$core".pcf
    mv nemo.row nemo_"$core".row	
    echo "Cutting best iteration"
    echo
    ./../src/magiccut/magicCut nemo_"${core}".prv "$Nemo_iterations" > cut_"$core".out 2>&1
    if ! ls nemo_"$core".best_cut.prv; then
      echo "Cut failed, look at Run_NEMO/cut_$core.out for more info." 
      echo
      exit 1
    fi
    echo
    # Creating folder
    
    if ! test -d "../Metrics"; then
      mkdir ../Metrics
    fi
    cp nemo_"$core".best_cut.* ../Metrics

    
    
  done

}

Create_metrics()
{
  
  
  cd "$dir" || (echo "Error original dir doesn't exist"; exit 1)
  # Create performance metrics
  echo "Creating metrics and storing theme in Metrics folder"
  echo
  python3 ./src/Job_Creator.py -f "analysis" -j "analysis" --set-core "${Jobs_n_cores}" --set-core-per-node "$Jobs_cores_per_node" -s "$Jobs_scheduler" --set-time "$time" --set-queue "$queue" -w "./../src/modelfactors.py -ms 100000 *" 
  mv analysis."$Jobs_scheduler" Metrics
  cd Metrics||(echo "Error Metrics folder doesn't exists";  exit 1)
  state5=$("$job" --wait analysis."$Jobs_scheduler")
  Job_completed "$state5"
  if [ $Completed == false ]; then
    echo "Error, metrics have not generated check Metrics/analysis.err to get more details"
    echo
    exit 1
 
  fi
  
  echo "------------------------------------------------------------------------------"
  echo "------------------------- Script Completed -----------------------------------"
  echo "----------------------- Data in Metrics folder -------------------------------"
  echo "------------------------------------------------------------------------------"
  echo "------------------------------------------------------------------------------"
  echo
}